import { __awaiter } from "tslib";
import { createClient } from 'redis';
/**
 * A Cache-Tags registry implemented using Redis.
 */
class RedisCacheTagsRegistry {
    constructor(config) {
        this.acting = 0;
        /**
         * Perform async actions on the store with ensured read/write.
         */
        this.act = (action) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            // Inform of ongoing action.
            this.acting++;
            // Ensure a closing connection ends it's process.
            yield this.disconnecting;
            // Establish connection
            yield (this.connecting = (_a = this.connecting) !== null && _a !== void 0 ? _a : this.client.connect());
            // Execute action.
            const result = yield action(this.client);
            // Only disconnect on last action.
            if (--this.acting === 0 && this.client.isOpen) {
                // Start disconnection process.
                this.disconnecting = this.client.quit();
                // Ensure new acts reconnect.
                delete this.connecting;
                // Wait for disconnection before proceeding
                yield this.disconnecting;
            }
            return result;
        });
        this.register = (path, tags) => __awaiter(this, void 0, void 0, function* () {
            return yield this.act(() => __awaiter(this, void 0, void 0, function* () {
                const now = new Date().toISOString();
                const transaction = this.client.multi();
                for (const tag of tags) {
                    // @ts-ignore
                    transaction.HSET(tag, path, now);
                }
                yield transaction.exec();
            }));
        });
        this.extract = (tag) => __awaiter(this, void 0, void 0, function* () {
            return yield this.act(() => __awaiter(this, void 0, void 0, function* () {
                var _b;
                // Retrieve all paths related to this key.
                const paths = Object.keys((_b = (yield this.client.HGETALL(tag))) !== null && _b !== void 0 ? _b : {});
                // Dispatch deletion.
                void this.client.DEL(tag);
                return paths;
            }));
        });
        this.client = createClient(config);
    }
}
export { RedisCacheTagsRegistry };
//# sourceMappingURL=redis.js.map